package vin.vio.collaboraonline.controller;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import vin.vio.collaboraonline.model.WopiFile;
import vin.vio.collaboraonline.service.FileService;

import java.io.File;
import java.io.InputStream;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @Description: TODO
 * @Date: 2025/2/19 16:49
 * @Created: by ZZSLL
 */

@Slf4j
@RestController
@RequestMapping("/wopi")
public class WopiController {

    @Autowired
    private FileService fileService;

    @Value("${collabora.host-server-addr}")
    private String host;

    // 检查文件信息
    @GetMapping("/files/{fileId}")
    public ResponseEntity<?> checkFileInfo(@PathVariable String fileId, @RequestParam(required = false) String access_token) {

        WopiFile file = fileService.getFileInfo(fileId);
        if (file == null) {
            return ResponseEntity.notFound().build();
        }

        return ResponseEntity.ok()
                .contentType(MediaType.APPLICATION_JSON)
                .body(createCheckFileInfoResponse(file));
    }

    // 获取文件内容
    @GetMapping("/files/{fileId}/contents")
    public ResponseEntity<Resource> getFile(@PathVariable String fileId) {

        Resource fileResource = fileService.loadFileAsResource(fileId);
        if (fileResource == null) {
            return ResponseEntity.notFound().build();
        }

        return ResponseEntity.ok()
                .header("Content-Type", "application/octet-stream")
                .body(fileResource);
    }

    // 保存文件
    @RequestMapping(value = "/files/{fileId}/contents", method = {RequestMethod.POST, RequestMethod.PUT})
    public ResponseEntity<Void> putFile(
            @PathVariable String fileId, InputStream contentStream) {

        boolean success = fileService.saveFile(fileId, contentStream);
        return success ?
                ResponseEntity.ok().build() :
                ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
    }

    private Map<String, Object> createCheckFileInfoResponse(WopiFile file) {
        Map<String, Object> response = new LinkedHashMap<>();

        // 强制开启所有编辑权限
        response.put("BaseFileName", file.getFileName());
        response.put("Size", file.getSize());
        response.put("OwnerId", "test_admin");  // 固定测试账户
        response.put("UserId", "test_user");    // 固定测试用户

        // 关键权限开关组
        response.put("UserCanWrite", true);     // 强制可写
        response.put("UserCanNotWriteRelative", false);
        response.put("SupportsLocks", false);   // 禁用文件锁检测
        response.put("SupportsUpdate", true);
        response.put("EnableOwnerTermination", true); // 允许强制终止锁

        // 关闭所有安全限制
        response.put("IsAnonymousUser", false);  // 允许匿名访问
        response.put("ProtectInClient", false); // 禁用客户端保护
        response.put("DisablePrint", false);
        response.put("DisableExport", false);
        response.put("DisableCopy", false);

        // 添加必要协议字段
        response.put("HostEditUrl", host + "/wopi/files/"+file.getFileName() + "?access_token=asdasd");
        response.put("PostMessageOrigin", host); // 本地测试用

        // 测试环境特殊配置
        response.put("BreadcrumbBrandName", "DEV ENV");
        response.put("BreadcrumbBrandUrl", host);
        response.put("BreadcrumbDocName", file.getFileName());

        // 绕过版本检查
//        response.put("LastModifiedTime", "2020-01-01T00:00:00Z");
//        response.put("Version", "test_version");

        return response;
    }
}