package vin.vio.collaboraonline.kingdee;


import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import vin.vio.collaboraonline.exception.AllKingdeeException;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Description: TODO
 * @Date: 2025/2/21 16:25
 * @Created: by ZZSLL
 */

@Service
public class AuthService {


    @Value("${ensign.kingdee.is-public}")
    private Boolean isPublic;

    @Value("${ensign.kingdee.test-public-end-point}")
    private String testPublicEndPoint;

    @Value("${ensign.kingdee.test-inner-end-point}")
    private String testInnerEndPoint;

    @Value("${ensign.kingdee.prod-public-end-point}")
    private String prodPublicEndPoint;

    @Value("${ensign.kingdee.prod-inner-end-point}")
    private String prodInnerEndPoint;

    @Value("${ensign.kingdee.env}")
    private String env;

    @Value("${ensign.kingdee.appToken.appId}")
    private String appId;

    @Value("${ensign.kingdee.appToken.appSecuret}")
    private String appSecuret;

    @Value("${ensign.kingdee.appToken.tenantid}")
    private String tenantid;

    @Value("${ensign.kingdee.appToken.accountId}")
    private String accountId;

    @Value("${ensign.kingdee.appToken.language}")
    private String language;

    @Value("${ensign.kingdee.accessToken.user}")
    private String user;

    @Value("${ensign.kingdee.accessToken.usertype}")
    private String usertype;

    public String initBasePath() {
        if ("test".equals(env)) {
            if (isPublic) {
                return testPublicEndPoint;
            } else {
                return testInnerEndPoint;
            }
        }

        if ("prod".equals(env)) {
            if (isPublic) {
                return prodPublicEndPoint;
            } else {
                return prodInnerEndPoint;
            }
        }

        return "";
    }



    public String initAccessToken() throws IOException, AllKingdeeException {

        final String END_POINT = initBasePath() + "/ierp/api/getAppToken.do";
        HttpClient client = new DefaultHttpClient();

        HttpPost post = new HttpPost(END_POINT);

        Map<String, String> body = new HashMap<>();

        body.put("appId", appId);
        body.put("appSecuret", appSecuret);
        body.put("tenantid", tenantid);
        body.put("accountId", accountId);
        body.put("language", language);

        HttpEntity reqEntity = new StringEntity(JSONUtil.toJsonStr(body), "utf-8");

        post.setEntity(reqEntity);
        post.setConfig(initRequestConfig());
        HttpResponse response = client.execute(post);
        if (response.getStatusLine().getStatusCode() == 200) {

            HttpEntity resEntity = response.getEntity();
            String responseData = EntityUtils.toString(resEntity, "utf-8");
            JSONObject jsonObject = JSON.parseObject(responseData);
            Boolean status = (Boolean) jsonObject.get("status");
            String state = (String) jsonObject.get("state");
            if (status && "success".equals(state)) {
                JSONObject data = (JSONObject) jsonObject.get("data");
                String appToken = (String) data.get("app_token");
                if (appToken != null && !appToken.isEmpty()) {
                    return doGetAccessToken(appToken);
                }
            }
        }
        throw new AllKingdeeException("获取AppToken失败");
    }

    private String doGetAccessToken(String appToken) throws IOException, AllKingdeeException {

        HttpClient client = new DefaultHttpClient();

        HttpPost post = new HttpPost(initBasePath() + "/ierp/api/login.do");

        Map<String, String> body = new HashMap<>();

        body.put("user", user);
        body.put("apptoken", appToken);
        body.put("tenantid", tenantid);
        body.put("accountId", accountId);
        body.put("language", language);
        body.put("usertype", usertype);

        HttpEntity reqEntity = new StringEntity(JSONUtil.toJsonStr(body), "utf-8");

        post.setEntity(reqEntity);
        post.setConfig(initRequestConfig());
        HttpResponse response = client.execute(post);

        if (response.getStatusLine().getStatusCode() == 200) {

            HttpEntity resEntity = response.getEntity();
            String responseData = EntityUtils.toString(resEntity, "utf-8");
            JSONObject jsonObject = JSON.parseObject(responseData);
            Boolean status = (Boolean) jsonObject.get("status");
            String state = (String) jsonObject.get("state");

            if (status && "success".equals(state)) {
                JSONObject data = (JSONObject) jsonObject.get("data");
                String accessToken = (String) data.get("access_token");
                if (accessToken != null && !accessToken.isEmpty()) {
                    return accessToken;
                }
            }
        }
        throw new AllKingdeeException("获取AccessToken失败");
    }

    private RequestConfig initRequestConfig() {
        return RequestConfig.custom()
                .setConnectTimeout(5000)//一、连接超时：connectionTimeout-->指的是连接一个url的连接等待时间
                .setSocketTimeout(5000)// 二、读取数据超时：SocketTimeout-->指的是连接上一个url，获取response的返回等待时间
                .setConnectionRequestTimeout(5000)
                .build();
    }

}
