package vin.vio.collaboraonline.service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.http.client.ClientHttpRequest;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import vin.vio.collaboraonline.exception.AllKingdeeException;
import vin.vio.collaboraonline.kingdee.AuthService;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

import static cn.hutool.http.HttpUtil.downloadFile;

/**
 * @Description: TODO
 * @Date: 2025/2/21 16:40
 * @Created: by ZZSLL
 */

@Service
public class KingdeeService {

    @Value("${collabora.file-storage-path}")
    private String BASE_DIR;

    @Autowired
    private AuthService authService;

    public String requestFile(String attPkId) {
        try {
            String accessToken = authService.initAccessToken();
            URI newUri = new URI(authService.initBasePath() + "/ierp/kapi/v2/yem/yem_receipt/AttachmentsQueryAPI/Query");

            ClientHttpRequest delegate = new SimpleClientHttpRequestFactory().createRequest(newUri, HttpMethod.POST);
            delegate.getHeaders().add("access_token", accessToken);
            delegate.getHeaders().add("Content-Type", "application/json;charset=UTF-8");
            Map<String, String> requestBody = new HashMap<>();
            requestBody.put("formId", "yem_testbill");
            requestBody.put("pkId", attPkId);
            requestBody.put("attachKey", "attachmentpanel");
            String jsonBody = JSON.toJSONString(requestBody);
            // 设置请求体
            try (OutputStream os = delegate.getBody()) {
                os.write(jsonBody.getBytes(StandardCharsets.UTF_8));
            }
            ClientHttpResponse response = delegate.execute();
            BufferedReader reader = new BufferedReader(new InputStreamReader(response.getBody()));
            StringBuilder responseBody = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                responseBody.append(line);
            }
            String savePath = "";
            JSONObject result = JSON.parseObject(responseBody.toString());
            if (result.containsKey("status") && result.getBooleanValue("status", false)) {
                JSONArray data = result.getJSONArray("data");
                if (data != null && !data.isEmpty()) {
                    JSONObject data0 = data.getJSONObject(0);
                    String url = data0.getString("url");
                    String name = data0.getString("name");
                    savePath = BASE_DIR + "/" + name;
                    downloadFile(url, savePath);
                }
            } else {
                throw new AllKingdeeException(line);
            }
            return savePath;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }
}
